function JointMortorUnified(joint, speed, direction)
    local min, max = GetJointLimits(joint)
    if GetJointType(joint) == "hinge" then
        if direction == "max" then
            SetJointMotorTarget(joint, max, speed)
        else
            SetJointMotorTarget(joint, min, speed)
        end
    elseif GetJointType(joint) == "prismatic" then
        if direction == "max" then
            SetJointMotorTarget(joint, max, speed/math.pi)
        else
            SetJointMotorTarget(joint, min, speed/math.pi)
        end
    end
end

function JointMortorLock(joint, speed, direction)
    local min, max = GetJointLimits(joint)
    if GetJointType(joint) == "hinge" then
        if direction == "max" then
            SetJointMotor(joint, 0, 0)
        end
        if direction == "min" then
            if GetJointMovement(joint) < 5 then
                SetJointMotorTarget(joint, min, speed)
            end
        end
    elseif GetJointType(joint) == "prismatic" then
        if direction == "max" then
            SetJointMotor(joint, 0, 0)
        end
        if direction == "min" then
            if GetJointMovement(joint) < 5 then
            SetJointMotorTarget(joint, max, speed/math.pi)
            end
        end
    end
end

function DoorYLVF(open, close, doors)
    if HasTag(GetPlayerPickBody(), "ALSdoor") then
        local door = GetPlayerPickBody()
        if InputPressed("grab") then
            if GetTagValue(door, "active") == open then
                SetTag(door, "active", close)
            elseif GetTagValue(door, "active") == close then
                SetTag(door, "active", open)
            end
        end
    end
    for i=1, #doors do
        local localdoors = GetBodyShapes(doors[i])
        if not HasTag(doors[i], "active") then
            SetTag(doors[i], "active", open)
        end
        for t=1, #localdoors do
            local doorJoints = GetShapeJoints(localdoors[t])
            if GetPlayerPickBody() == doors[i] or IsBodyActive(doors[i]) then
                if GetTagValue(doors[i], "active") == open then
                    for j=1, #doorJoints do
                        JointMortorUnified(doorJoints[j], 2, "min")
                    end
                elseif GetTagValue(doors[i], "active") == close then
                    RemoveTag(doors[i], "activedriver")
                    for j=1, #doorJoints do
                        JointMortorUnified(doorJoints[j], 2, "max")
                    end
                end
            end
        end
    end
end

function DoorClick(doors)   
    if HasTag(GetPlayerPickBody(), "ALSdoor") then
        local door = GetPlayerPickBody()
        if InputPressed("grab") then
            if GetTagValue(door, "active") then
                SetTag(door, "forceopen")
                RemoveTag(door, "openonce")
            end
        end
    end
        if InputDown("Grab") then
            forceopen = true
        else
            forceopen = false
        end
    for i=1, #doors do
        local localdoors = GetBodyShapes(doors[i])
        for t=1, #localdoors do
            local doorJoints = GetShapeJoints(localdoors[t])
            if GetPlayerPickBody() == doors[i] or IsBodyActive(doors[i]) then
                if GetTagValue(doors[i], "active") and HasTag(doors[i], "forceopen") then
                    for j=1, #doorJoints do
                        JointMortorLock(doorJoints[j], 2, "max")
                    end
                    if not forceopen then
                        RemoveTag(doors[i], "forceopen")
                    end
                elseif GetTagValue(doors[i], "active") and not HasTag(doors[i], "forceopen") then
                    for j=1, #doorJoints do
                        JointMortorLock(doorJoints[j], 2, "min")
                    end
                end
            end
        end
    end
end

function DoorExit(open, close, doors)
    for i=1, #doors do
        local localdoors = GetBodyShapes(doors[i]) 
        if not HasTag(doors[i], "active") then
            SetTag(doors[i], "active", open)
        end
        for t=1, #localdoors do
            local doorJoints = GetShapeJoints(localdoors[t])
            if GetPlayerVehicle() == GetBodyVehicle(doors[i]) and GetPlayerVehicle() ~= 0 then
                for j=1, #doorJoints do
                    JointMortorUnified(doorJoints[j], 2, "min")
                end
                if GetTagValue(doors[i], "ALSdoor") == "driver" then
                    SetTag(doors[i], "active", close)
                    SetTag(doors[i], "activedriver")
                    SetTag(doors[i], "openonce")
                else
                    SetTag(doors[i], "active", open)
                end
            elseif GetTagValue(doors[i], "active") == close and HasTag(doors[i], "openonce") then
                local min, max = GetJointLimits(doorJoints[1])
                if GetJointMovement(doorJoints[1]) <= max then
                    RemoveTag(doors[i], "activedriver")
                    for j=1, #doorJoints do
                        JointMortorUnified(doorJoints[j], 2, "max")
                    end
                end
            end
        end
    end
end

function honkNsiren(sirenShapes, sirens1, sirens2, sirens3, honks, ctrl)
    if sirenShapes ~= empty then
        for i=1, #sirenShapes do
            if not HasTag(sirenShapes[i], "active")  then
                SetTag(sirenShapes[i], "active")
            end
            if InputDown(ctrl[4]) and GetPlayerVehicle() == GetBodyVehicle(GetShapeBody(sirenShapes[i])) then
                local x, y, z = GetShapeSize(sirenShapes[i])
                local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                local honkNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 1, 2))
                PlayLoop(honks[honkNum], honkCentre.pos)
            else
                if not HasTag(sirenShapes[i], "ALSsiren1") and not HasTag(sirenShapes[i], "ALSsiren2") and not HasTag(sirenShapes[i], "ALSsiren3") then
                    if InputDown(ctrl[5]) and not InputDown(ctrl[6]) and GetPlayerVehicle() == GetBodyVehicle(GetShapeBody(sirenShapes[i]))  then
                        local x, y, z = GetShapeSize(sirenShapes[i])
                        local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                        local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 3, 4))
                        PlayLoop(sirens1[sirenNum], honkCentre.pos)
                    end
                    if InputDown(ctrl[5]) and InputDown(ctrl[6]) and GetPlayerVehicle() == GetBodyVehicle(GetShapeBody(sirenShapes[i]))  then
                        local x, y, z = GetShapeSize(sirenShapes[i])
                        local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                        local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 5, 6))
                        PlayLoop(sirens2[sirenNum], honkCentre.pos)
                    end
                end
                if HasTag(sirenShapes[i], "ALSsiren1") and not HasTag(sirenShapes[i], "Yell") then
                    local x, y, z = GetShapeSize(sirenShapes[i])
                    local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                    local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 3, 4))
                    PlayLoop(sirens1[sirenNum], honkCentre.pos)
                end
                if HasTag(sirenShapes[i], "ALSsiren2") and not HasTag(sirenShapes[i], "Yell") then
                    local x, y, z = GetShapeSize(sirenShapes[i])
                    local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                    local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 5, 6))
                    PlayLoop(sirens2[sirenNum], honkCentre.pos)
                end
                 if HasTag(sirenShapes[i], "ALSsiren3") and not HasTag(sirenShapes[i], "Yell") then
                    local x, y, z = GetShapeSize(sirenShapes[i])
                    local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                    local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 7, 8))
                    PlayLoop(sirens3[sirenNum], honkCentre.pos)
                end
                if HasTag(sirenShapes[i], "ALSsiren1") and HasTag(sirenShapes[i], "Yell") then
                    local x, y, z = GetShapeSize(sirenShapes[i])
                    local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                    local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 5, 6))
                    PlayLoop(sirens2[sirenNum], honkCentre.pos)
                end
                if HasTag(sirenShapes[i], "ALSsiren2") and HasTag(sirenShapes[i], "Yell") then
                    local x, y, z = GetShapeSize(sirenShapes[i])
                    local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                    local sirenNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 7, 8))
                    PlayLoop(sirens3[sirenNum], honkCentre.pos)
                end
                if HasTag(sirenShapes[i], "ALSsiren3") and HasTag(sirenShapes[i], "Yell") then
                    local x, y, z = GetShapeSize(sirenShapes[i])
                    local honkCentre = TransformToParentTransform(GetShapeWorldTransform(sirenShapes[i]), Transform(Vec(x/20, y/20, z/20)))
                    local honkNum = tonumber(string.sub(GetTagValue(sirenShapes[i], "ALShorn"), 1, 2))
                    PlayLoop(honks[honkNum], honkCentre.pos)
                end
                if string.len(GetTagValue(sirenShapes[i], "ALShorn")) == 8 then
                    if GetPlayerVehicle() == GetBodyVehicle(GetShapeBody(sirenShapes[i])) then
                        if InputDown(ctrl[5]) then
                            SetTag(sirenShapes[i], "Yell")
                        else
                            RemoveTag(sirenShapes[i], "Yell")
                        end
                        if InputPressed(ctrl[1]) then
                            if HasTag(sirenShapes[i], "ALSsiren1") then
                                RemoveTag(sirenShapes[i], "ALSsiren1")
                                RemoveTag(sirenShapes[i], "ALSsiren2")
                                RemoveTag(sirenShapes[i], "ALSsiren3")
                            else
                                SetTag(sirenShapes[i], "ALSsiren1")
                                RemoveTag(sirenShapes[i], "ALSsiren2")
                                RemoveTag(sirenShapes[i], "ALSsiren3")
                            end
                        end
                        if InputPressed(ctrl[2]) then
                            if HasTag(sirenShapes[i], "ALSsiren2") then
                                RemoveTag(sirenShapes[i], "ALSsiren1")
                                RemoveTag(sirenShapes[i], "ALSsiren2")
                                RemoveTag(sirenShapes[i], "ALSsiren3")
                            else
                                SetTag(sirenShapes[i], "ALSsiren2")
                                RemoveTag(sirenShapes[i], "ALSsiren1")
                                RemoveTag(sirenShapes[i], "ALSsiren3")
                            end
                        end
                        if InputPressed(ctrl[3]) then
                            if HasTag(sirenShapes[i], "ALSsiren3") then
                                RemoveTag(sirenShapes[i], "ALSsiren1")
                                RemoveTag(sirenShapes[i], "ALSsiren2")
                                RemoveTag(sirenShapes[i], "ALSsiren3")
                            else
                                SetTag(sirenShapes[i], "ALSsiren3")
                                RemoveTag(sirenShapes[i], "ALSsiren1")
                                RemoveTag(sirenShapes[i], "ALSsiren2")
                            end
                        end
                    end
                end
            end
        end
    end
end

function beepclik(sirenShapes, click, ctrl)
    if sirenShapes ~= empty then
        for i=1, #sirenShapes do
            if GetBool(pathUi .. "FPS") then
                SetTag(sirenShapes[i], "delete")
            end
            if GetPlayerVehicle() == GetBodyVehicle(GetShapeBody(sirenShapes[i])) then
                if HasTag(sirenShapes[i], "emergency") then
                    em = GetTagValue(sirenShapes[i], "em")
                    if InputPressed(ctrl[9]) then
                        if em == "1" then
                            RemoveTag(sirenShapes[i], "em")
                            RemoveTag(sirenShapes[i], "ALSsiren1")
                            RemoveTag(sirenShapes[i], "ALSsiren2")
                            RemoveTag(sirenShapes[i], "ALSsiren3")
                        else
                            SetTag(sirenShapes[i], "em", "1")
                        end
                    end
                    if InputPressed(ctrl[10]) then
                        if em == "2" then
                            RemoveTag(sirenShapes[i], "em")
                            RemoveTag(sirenShapes[i], "ALSsiren1")
                            RemoveTag(sirenShapes[i], "ALSsiren2")
                            RemoveTag(sirenShapes[i], "ALSsiren3")
                        else
                            SetTag(sirenShapes[i], "em", "2")
                        end
                    end
                    if InputPressed(ctrl[11]) then
                        if em == "3" then
                            RemoveTag(sirenShapes[i], "em")
                            RemoveTag(sirenShapes[i], "ALSsiren1")
                            RemoveTag(sirenShapes[i], "ALSsiren2")
                            RemoveTag(sirenShapes[i], "ALSsiren3")
                        else
                            SetTag(sirenShapes[i], "em", "3")
                        end
                    end
                    if em == "1" and InputPressed(ctrl[9]) or em == "2" and InputPressed(ctrl[10]) or em == "3" and InputPressed(ctrl[11]) then
                        PlaySound(click[02],GetCameraTransform().pos,0.5)    
                    else if InputPressed(ctrl[9]) or InputPressed(ctrl[10]) or InputPressed(ctrl[11]) then
                        PlaySound(click[01],GetCameraTransform().pos,0.5)
                        end
                    end
                end
                if HasTag(sirenShapes[i], "directional") then
                    di = GetTagValue(sirenShapes[i], "di")
                    if InputPressed(ctrl[12]) then
                        if di == "1" then
                            RemoveTag(sirenShapes[i], "di")
                        else
                            SetTag(sirenShapes[i], "di", "1")
                        end
                    end

                    if di == "1" and InputPressed(ctrl[12]) then
                        PlaySound(click[02],GetCameraTransform().pos,0.5)    
                    else if InputPressed(ctrl[12]) then
                        PlaySound(click[01],GetCameraTransform().pos,0.5)
                        end
                    end
                end

                p = GetTagValue(sirenShapes[i], "p")
                l = GetTagValue(sirenShapes[i], "l")
                la = GetTagValue(sirenShapes[i], "la")
                lat = GetTagValue(sirenShapes[i], "lat")
                fa = GetTagValue(sirenShapes[i], "fa")

                if InputPressed(ctrl[8]) then
                    PlaySound(click[03],GetCameraTransform().pos,0.8)
                    if p == "1" then
                        RemoveTag(sirenShapes[i], "p")
                    else
                        SetTag(sirenShapes[i], "p", "1")
                    end
                end
                if InputPressed(ctrl[7]) then
                    PlaySound(click[03],GetCameraTransform().pos,0.8)
                    if p == "2" then
                        RemoveTag(sirenShapes[i], "p")
                    else
                        SetTag(sirenShapes[i], "p", "2")
                    end
                end
                if InputPressed(ctrl[5]) then
                    PlaySound(click[03],GetCameraTransform().pos,0.8)
                    if p == "3" then
                        RemoveTag(sirenShapes[i], "p")
                    else
                        SetTag(sirenShapes[i], "p", "3")
                    end
                end

                if InputPressed(ctrl[1]) then
                    PlaySound(click[03],GetCameraTransform().pos,0.8)
                    if l == "1" then
                        RemoveTag(sirenShapes[i], "l")
                    else
                        SetTag(sirenShapes[i], "l", "1")
                    end
                end
                if InputPressed(ctrl[2]) then
                    PlaySound(click[03],GetCameraTransform().pos,0.8)
                    if la == "1" then
                        RemoveTag(sirenShapes[i], "la")
                    else
                        SetTag(sirenShapes[i], "la", "1")
                    end
                end
                if InputDown(ctrl[3]) then
                    SetTag(sirenShapes[i], "lat", "1")
                else
                    RemoveTag(sirenShapes[i], "lat")
                end

                if InputPressed(ctrl[4]) then
                    PlaySound(click[03],GetCameraTransform().pos,0.8)
                    if fa == "1" then
                        RemoveTag(sirenShapes[i], "fa")
                    else
                        SetTag(sirenShapes[i], "fa", "1")
                    end
                end
            end
        end
    end
end

function Noclip(noclip)
    for i=1, #noclip do
    SetShapeCollisionFilter(noclip[i], 2, 255-2)
    end
end

function FontList(font)
    local newFont = ""
    if font == 1 then newFont = "regular.ttf" end
    if font == 2 then newFont = "bold.ttf" end
    return newFont
end

function DrawColoredTextButtons(condition, text, w, h, font, size, R, G, B)
    local newFont = FontList(font)
    UiPush()
        if condition then
            UiColor(R, G, B)
            UiButtonHoverColor(R*10, G*0.6, B*0.2)
        else
            UiColor(1, 1, 1)
            UiButtonHoverColor(R*0.6, G, B*0.6)
        end
        UiButtonPressDist(3)
        UiFont(newFont, size)
        UiButtonImageBox("images/box-outline-4.png", 6, 6)
        if UiTextButton(text, w, h) then
            condition = not condition
            press = true
        else
            press = false
        end
    UiPop()
    return press, condition
end


function DrawDynamicTextButtons(condition, text1, text2, w, h, font, size, R, G, B)
    local newFont = FontList(font)
    local press = false
    UiPush()
        UiButtonPressDist(3)
        UiFont(newFont, size)
        UiButtonImageBox("images/box-outline-8.png", 6, 6)
        if condition then
            UiColor(R, G, B)
            UiButtonHoverColor(R*10, G*0.6, B*0.2)
            if UiTextButton(text2, w, h) then
                condition = not condition
                press = true
            end
        else
            UiColor(1, 1, 1)
            UiButtonHoverColor(R*0.6, G, B*0.6)
            if UiTextButton(text1, w, h) then
                condition = not condition
                press = true
            end
        end
    UiPop()
    return press, condition
end


function scrollAndDrag(windowW, windowH, contentLength, scrollSpeed, adjustFactor, background)

    local length = contentLength or 1
    local inputScrollFactor = scrollSpeed or 1
    local adjust = adjustFactor or 1

    local wheelMove = false
    local wheelMoveTime = 0
    local tempWheel = 0

    UiAlign("top left")
    UiWindow(windowW, windowH, true)
    UiPush()
        UiTranslate(-1, -1)
        local alpha = background[4] or 1
        if background[1] > 1 or background[2] > 1 or background[3] > 1 or alpha > 1 then
            if background[4] == nil then alpha = 100 end
            UiColor(background[1]/255, background[2]/255, background[3]/255, alpha/100)
        else
            if background[4] == nil then alpha = 1 end
            UiColor(background[1], background[2], background[3], alpha)
        end
        UiRect(windowW+2, windowH+2)
    UiPop()
    local factor = length/(windowH-20)
    if UiIsMouseInRect(windowW, windowH) then
        if InputValue("mousewheel") ~= 0 then
            wheelMove = true
            tempWheel = InputValue("mousewheel")
            wheelMoveTime = GetTime()
        else
            if (GetTime()-wheelMoveTime) > 0.5 then
                wheelMove = false
            end
        end
        if wheelMove then
            SetValue("wheel", tempWheel*inputScrollFactor)
        else
            SetValue("wheel", 0)
        end
    end
    if factor >= 1 then
        local newslide = slide - wheel*0.3
        SetValue("slide", newslide, linear, 2)
        if slide >= ((windowH-20)/20)*factor then slide = ((windowH-20)/20)*factor end
        if slide <= 0 then slide = 0 end
        UiPush()
            UiTranslate(windowW-10, 0)
            newslide, dragDone = DrawSliderVirtical(slide, ((windowH-20)/20)*factor, windowH, 0.2, 0.2, 0.2, 1, 1, 1)
        UiPop()
        SetValue("slide", newslide, linear, 2)
        if slide >= ((windowH-20)/20)*factor then slide = ((windowH-20)/20)*factor end
        if slide <= 0 then slide = 0 end
    end
    local slideOutput = -slide*adjust
    if factor < 1 then slideOutput = 0 end
    return slideOutput
end

function DrawSlider(value, length, R, G, B, dotR, dotG, dotB)
    UiPush()
        UiAlign("left middle")
        UiColor(R, G, B)
        UiRect(length+20, 20)
        UiColor(dotR, dotG, dotB)
        slidValue, done = UiSlider("ui/common/box-solid-4.png", x, value, 0, length)
    UiPop()
    return slidValue, done
end

function DrawSliderVirtical(value, length, screenLength, R, G, B, dotR, dotG, dotB)
    UiPush()
        UiScale(1, screenLength/(length+20))
        UiRotate(-90)
        UiAlign("left middle")
        UiColor(R, G, B)
        UiRect(length+20, 20)
        UiColor(dotR, dotG, dotB)
        slidValue, done = UiSlider("images/slider-vertical.png", x, value, 0, length)
    UiPop()
    return slidValue, done
end